@@ -6,42 +6,37 @@ A Cairo backend for matplotlib
 This backend depends on cairocffi or pycairo.
 """
 
-import copy
 import gzip
-import sys
-import warnings
+import math
 
 import numpy as np
 
-# cairocffi is more widely compatible than pycairo (in particular pgi only
-# works with cairocffi) so try it first.
 try:
-    import cairocffi as cairo
+    import cairo
+    if cairo.version_info < (1, 11, 0):
+        # Introduced create_for_data for Py3.
+        raise ImportError
 except ImportError:
     try:
-        import cairo
-    except ImportError:
-        raise ImportError("cairo backend requires that cairocffi or pycairo "
-                          "is installed")
-    else:
-        if cairo.version_info < (1, 11, 0):
-            # Introduced create_for_data for Py3.
-            raise ImportError(
-                "cairo {} is installed; cairo>=1.11.0 is required"
-                .format(cairo.version))
+        import cairocffi as cairo
+    except ImportError as err:
+        raise ImportError(
+            "cairo backend requires that pycairo>=1.11.0 or cairocffi "
+            "is installed") from err
 
-backend_version = cairo.version
-
-from .. import cbook
+from .. import _api, cbook, font_manager
 from matplotlib.backend_bases import (
-    _Backend, FigureCanvasBase, FigureManagerBase, GraphicsContextBase,
-    RendererBase)
+    _Backend, _check_savefig_extra_args, FigureCanvasBase, FigureManagerBase,
+    GraphicsContextBase, RendererBase)
 from matplotlib.font_manager import ttfFontProperty
 from matplotlib.mathtext import MathTextParser
 from matplotlib.path import Path
 from matplotlib.transforms import Affine2D
 
 
+backend_version = cairo.version
+
+
 if cairo.__name__ == "cairocffi":
     # Convert a pycairo context to a cairocffi one.
     def _to_context(ctx):
@@ -58,147 +53,85 @@ else:
         return ctx
 
 
-@cbook.deprecated("3.0")
-class ArrayWrapper:
-    """Thin wrapper around numpy ndarray to expose the interface
-       expected by cairocffi. Basically replicates the
-       array.array interface.
-    """
-    def __init__(self, myarray):
-        self.__array = myarray
-        self.__data = myarray.ctypes.data
-        self.__size = len(myarray.flatten())
-        self.itemsize = myarray.itemsize
-
-    def buffer_info(self):
-        return (self.__data, self.__size)
-
-
-# Mapping from Matplotlib Path codes to cairo path codes.
-_MPL_TO_CAIRO_PATH_TYPE = np.zeros(80, dtype=int)  # CLOSEPOLY = 79.
-_MPL_TO_CAIRO_PATH_TYPE[Path.MOVETO] = cairo.PATH_MOVE_TO
-_MPL_TO_CAIRO_PATH_TYPE[Path.LINETO] = cairo.PATH_LINE_TO
-_MPL_TO_CAIRO_PATH_TYPE[Path.CURVE4] = cairo.PATH_CURVE_TO
-_MPL_TO_CAIRO_PATH_TYPE[Path.CLOSEPOLY] = cairo.PATH_CLOSE_PATH
-# Sizes in cairo_path_data_t of each cairo path element.
-_CAIRO_PATH_TYPE_SIZES = np.zeros(4, dtype=int)
-_CAIRO_PATH_TYPE_SIZES[cairo.PATH_MOVE_TO] = 2
-_CAIRO_PATH_TYPE_SIZES[cairo.PATH_LINE_TO] = 2
-_CAIRO_PATH_TYPE_SIZES[cairo.PATH_CURVE_TO] = 4
-_CAIRO_PATH_TYPE_SIZES[cairo.PATH_CLOSE_PATH] = 1
-
-
-def _append_paths_slow(ctx, paths, transforms, clip=None):
-    for path, transform in zip(paths, transforms):
-        for points, code in path.iter_segments(
-                transform, remove_nans=True, clip=clip):
-            if code == Path.MOVETO:
-                ctx.move_to(*points)
-            elif code == Path.CLOSEPOLY:
-                ctx.close_path()
-            elif code == Path.LINETO:
-                ctx.line_to(*points)
-            elif code == Path.CURVE3:
-                cur = np.asarray(ctx.get_current_point())
-                a = points[:2]
-                b = points[-2:]
-                ctx.curve_to(*(cur / 3 + a * 2 / 3), *(a * 2 / 3 + b / 3), *b)
-            elif code == Path.CURVE4:
-                ctx.curve_to(*points)
-
-
-def _append_paths_fast(ctx, paths, transforms, clip=None):
-    # We directly convert to the internal representation used by cairo, for
-    # which ABI compatibility is guaranteed.  The layout for each item is
-    # --CODE(4)--  -LENGTH(4)-  ---------PAD(8)---------
-    # ----------X(8)----------  ----------Y(8)----------
-    # with the size in bytes in parentheses, and (X, Y) repeated as many times
-    # as there are points for the current code.
-    ffi = cairo.ffi
-
-    # Convert curves to segment, so that 1. we don't have to handle
-    # variable-sized CURVE-n codes, and 2. we don't have to implement degree
-    # elevation for quadratic Beziers.
-    cleaneds = [path.cleaned(transform, remove_nans=True, clip=clip)
-                for path, transform in zip(paths, transforms)]
-    vertices = np.concatenate([cleaned.vertices for cleaned in cleaneds])
-    codes = np.concatenate([cleaned.codes for cleaned in cleaneds])
-
-    # Remove unused vertices and convert to cairo codes.  Note that unlike
-    # cairo_close_path, we do not explicitly insert an extraneous MOVE_TO after
-    # CLOSE_PATH, so our resulting buffer may be smaller.
-    vertices = vertices[(codes != Path.STOP) & (codes != Path.CLOSEPOLY)]
-    codes = codes[codes != Path.STOP]
-    codes = _MPL_TO_CAIRO_PATH_TYPE[codes]
-
-    # Where are the headers of each cairo portions?
-    cairo_type_sizes = _CAIRO_PATH_TYPE_SIZES[codes]
-    cairo_type_positions = np.insert(np.cumsum(cairo_type_sizes), 0, 0)
-    cairo_num_data = cairo_type_positions[-1]
-    cairo_type_positions = cairo_type_positions[:-1]
-
-    # Fill the buffer.
-    buf = np.empty(cairo_num_data * 16, np.uint8)
-    as_int = np.frombuffer(buf.data, np.int32)
-    as_int[::4][cairo_type_positions] = codes
-    as_int[1::4][cairo_type_positions] = cairo_type_sizes
-    as_float = np.frombuffer(buf.data, np.float64)
-    mask = np.ones_like(as_float, bool)
-    mask[::2][cairo_type_positions] = mask[1::2][cairo_type_positions] = False
-    as_float[mask] = vertices.ravel()
-
-    # Construct the cairo_path_t, and pass it to the context.
-    ptr = ffi.new("cairo_path_t *")
-    ptr.status = cairo.STATUS_SUCCESS
-    ptr.data = ffi.cast("cairo_path_data_t *", ffi.from_buffer(buf))
-    ptr.num_data = cairo_num_data
-    cairo.cairo.cairo_append_path(ctx._pointer, ptr)
-
-
-_append_paths = (_append_paths_fast if cairo.__name__ == "cairocffi"
-                 else _append_paths_slow)
-
-
 def _append_path(ctx, path, transform, clip=None):
-    return _append_paths(ctx, [path], [transform], clip)
+    for points, code in path.iter_segments(
+            transform, remove_nans=True, clip=clip):
+        if code == Path.MOVETO:
+            ctx.move_to(*points)
+        elif code == Path.CLOSEPOLY:
+            ctx.close_path()
+        elif code == Path.LINETO:
+            ctx.line_to(*points)
+        elif code == Path.CURVE3:
+            cur = np.asarray(ctx.get_current_point())
+            a = points[:2]
+            b = points[-2:]
+            ctx.curve_to(*(cur / 3 + a * 2 / 3), *(a * 2 / 3 + b / 3), *b)
+        elif code == Path.CURVE4:
+            ctx.curve_to(*points)
+
+
+def _cairo_font_args_from_font_prop(prop):
+    """
+    Convert a `.FontProperties` or a `.FontEntry` to arguments that can be
+    passed to `.Context.select_font_face`.
+    """
+    def attr(field):
+        try:
+            return getattr(prop, f"get_{field}")()
+        except AttributeError:
+            return getattr(prop, field)
+
+    name = attr("name")
+    slant = getattr(cairo, f"FONT_SLANT_{attr('style').upper()}")
+    weight = attr("weight")
+    weight = (cairo.FONT_WEIGHT_NORMAL
+              if font_manager.weight_dict.get(weight, weight) < 550
+              else cairo.FONT_WEIGHT_BOLD)
+    return name, slant, weight
+
+
+# Mappings used for deprecated properties in RendererCairo, see below.
+_f_weights = {
+    100:          cairo.FONT_WEIGHT_NORMAL,
+    200:          cairo.FONT_WEIGHT_NORMAL,
+    300:          cairo.FONT_WEIGHT_NORMAL,
+    400:          cairo.FONT_WEIGHT_NORMAL,
+    500:          cairo.FONT_WEIGHT_NORMAL,
+    600:          cairo.FONT_WEIGHT_BOLD,
+    700:          cairo.FONT_WEIGHT_BOLD,
+    800:          cairo.FONT_WEIGHT_BOLD,
+    900:          cairo.FONT_WEIGHT_BOLD,
+    'ultralight': cairo.FONT_WEIGHT_NORMAL,
+    'light':      cairo.FONT_WEIGHT_NORMAL,
+    'normal':     cairo.FONT_WEIGHT_NORMAL,
+    'medium':     cairo.FONT_WEIGHT_NORMAL,
+    'regular':    cairo.FONT_WEIGHT_NORMAL,
+    'semibold':   cairo.FONT_WEIGHT_BOLD,
+    'bold':       cairo.FONT_WEIGHT_BOLD,
+    'heavy':      cairo.FONT_WEIGHT_BOLD,
+    'ultrabold':  cairo.FONT_WEIGHT_BOLD,
+    'black':      cairo.FONT_WEIGHT_BOLD,
+}
+_f_angles = {
+    'italic':  cairo.FONT_SLANT_ITALIC,
+    'normal':  cairo.FONT_SLANT_NORMAL,
+    'oblique': cairo.FONT_SLANT_OBLIQUE,
+}
 
 
 class RendererCairo(RendererBase):
-    fontweights = {
-        100          : cairo.FONT_WEIGHT_NORMAL,
-        200          : cairo.FONT_WEIGHT_NORMAL,
-        300          : cairo.FONT_WEIGHT_NORMAL,
-        400          : cairo.FONT_WEIGHT_NORMAL,
-        500          : cairo.FONT_WEIGHT_NORMAL,
-        600          : cairo.FONT_WEIGHT_BOLD,
-        700          : cairo.FONT_WEIGHT_BOLD,
-        800          : cairo.FONT_WEIGHT_BOLD,
-        900          : cairo.FONT_WEIGHT_BOLD,
-        'ultralight' : cairo.FONT_WEIGHT_NORMAL,
-        'light'      : cairo.FONT_WEIGHT_NORMAL,
-        'normal'     : cairo.FONT_WEIGHT_NORMAL,
-        'medium'     : cairo.FONT_WEIGHT_NORMAL,
-        'regular'    : cairo.FONT_WEIGHT_NORMAL,
-        'semibold'   : cairo.FONT_WEIGHT_BOLD,
-        'bold'       : cairo.FONT_WEIGHT_BOLD,
-        'heavy'      : cairo.FONT_WEIGHT_BOLD,
-        'ultrabold'  : cairo.FONT_WEIGHT_BOLD,
-        'black'      : cairo.FONT_WEIGHT_BOLD,
-                   }
-    fontangles = {
-        'italic'  : cairo.FONT_SLANT_ITALIC,
-        'normal'  : cairo.FONT_SLANT_NORMAL,
-        'oblique' : cairo.FONT_SLANT_OBLIQUE,
-        }
-
+    fontweights = _api.deprecated("3.3")(property(lambda self: {*_f_weights}))
+    fontangles = _api.deprecated("3.3")(property(lambda self: {*_f_angles}))
+    mathtext_parser = _api.deprecated("3.4")(
+        property(lambda self: MathTextParser('Cairo')))
 
     def __init__(self, dpi):
         self.dpi = dpi
         self.gc = GraphicsContextCairo(renderer=self)
         self.text_ctx = cairo.Context(
            cairo.ImageSurface(cairo.FORMAT_ARGB32, 1, 1))
-        self.mathtext_parser = MathTextParser('Cairo')
-        RendererBase.__init__(self)
+        super().__init__()
 
     def set_ctx_from_surface(self, surface):
         self.gc.ctx = cairo.Context(surface)
@@ -208,7 +141,7 @@ class RendererCairo(RendererBase):
         # for PDF/PS/SVG surfaces, which have no way to report their extents.
 
     def set_width_height(self, width, height):
-        self.width  = width
+        self.width = width
         self.height = height
 
     def _fill_and_stroke(self, ctx, fill_c, alpha, alpha_overrides):
@@ -222,12 +155,8 @@ class RendererCairo(RendererBase):
             ctx.restore()
         ctx.stroke()
 
-    @staticmethod
-    @cbook.deprecated("3.0")
-    def convert_path(ctx, path, transform, clip=None):
-        _append_path(ctx, path, transform, clip)
-
     def draw_path(self, gc, path, transform, rgbFace=None):
+        # docstring inherited
         ctx = gc.ctx
         # Clip the path to the actual rendering extents if it isn't filled.
         clip = (ctx.clip_extents()
@@ -242,8 +171,9 @@ class RendererCairo(RendererBase):
 
     def draw_markers(self, gc, marker_path, marker_trans, path, transform,
                      rgbFace=None):
-        ctx = gc.ctx
+        # docstring inherited
 
+        ctx = gc.ctx
         ctx.new_path()
         # Create the path for the marker; it needs to be flipped here already!
         _append_path(ctx, marker_path, marker_trans + Affine2D().scale(1, -1))
@@ -301,7 +231,9 @@ class RendererCairo(RendererBase):
         ctx.restore()
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath=False, mtext=None):
-        # Note: x,y are device/display coords, not user-coords, unlike other
+        # docstring inherited
+
+        # Note: (x, y) are device/display coords, not user-coords, unlike other
         # draw_* methods
         if ismath:
             self._draw_mathtext(gc, x, y, s, prop, angle)
@@ -310,74 +242,65 @@ class RendererCairo(RendererBase):
             ctx = gc.ctx
             ctx.new_path()
             ctx.move_to(x, y)
-            ctx.select_font_face(prop.get_name(),
-                                 self.fontangles[prop.get_style()],
-                                 self.fontweights[prop.get_weight()])
-
-            size = prop.get_size_in_points() * self.dpi / 72.0
 
+            ctx.select_font_face(*_cairo_font_args_from_font_prop(prop))
             ctx.save()
+            ctx.set_font_size(prop.get_size_in_points() * self.dpi / 72)
             if angle:
                 ctx.rotate(np.deg2rad(-angle))
-            ctx.set_font_size(size)
-
             ctx.show_text(s)
             ctx.restore()
 
     def _draw_mathtext(self, gc, x, y, s, prop, angle):
         ctx = gc.ctx
-        width, height, descent, glyphs, rects = self.mathtext_parser.parse(
-            s, self.dpi, prop)
+        width, height, descent, glyphs, rects = \
+            self._text2path.mathtext_parser.parse(s, self.dpi, prop)
 
         ctx.save()
         ctx.translate(x, y)
         if angle:
             ctx.rotate(np.deg2rad(-angle))
 
-        for font, fontsize, s, ox, oy in glyphs:
+        for font, fontsize, idx, ox, oy in glyphs:
             ctx.new_path()
-            ctx.move_to(ox, oy)
-
-            fontProp = ttfFontProperty(font)
-            ctx.select_font_face(fontProp.name,
-                                 self.fontangles[fontProp.style],
-                                 self.fontweights[fontProp.weight])
-
-            size = fontsize * self.dpi / 72.0
-            ctx.set_font_size(size)
-            ctx.show_text(s)
+            ctx.move_to(ox, -oy)
+            ctx.select_font_face(
+                *_cairo_font_args_from_font_prop(ttfFontProperty(font)))
+            ctx.set_font_size(fontsize * self.dpi / 72)
+            ctx.show_text(chr(idx))
 
         for ox, oy, w, h in rects:
             ctx.new_path()
-            ctx.rectangle(ox, oy, w, h)
+            ctx.rectangle(ox, -oy, w, -h)
             ctx.set_source_rgb(0, 0, 0)
             ctx.fill_preserve()
 
         ctx.restore()
 
     def get_canvas_width_height(self):
+        # docstring inherited
         return self.width, self.height
 
     def get_text_width_height_descent(self, s, prop, ismath):
+        # docstring inherited
+
+        if ismath == 'TeX':
+            return super().get_text_width_height_descent(s, prop, ismath)
+
         if ismath:
-            width, height, descent, fonts, used_characters = \
-                self.mathtext_parser.parse(s, self.dpi, prop)
+            width, height, descent, *_ = \
+                self._text2path.mathtext_parser.parse(s, self.dpi, prop)
             return width, height, descent
 
         ctx = self.text_ctx
-        ctx.save()
-        ctx.select_font_face(prop.get_name(),
-                             self.fontangles[prop.get_style()],
-                             self.fontweights[prop.get_weight()])
-
-        # Cairo (says it) uses 1/96 inch user space units, ref: cairo_gstate.c
-        # but if /96.0 is used the font is too small
-        size = prop.get_size_in_points() * self.dpi / 72
-
         # problem - scale remembers last setting and font can become
         # enormous causing program to crash
         # save/restore prevents the problem
-        ctx.set_font_size(size)
+        ctx.save()
+        ctx.select_font_face(*_cairo_font_args_from_font_prop(prop))
+        # Cairo (says it) uses 1/96 inch user space units, ref: cairo_gstate.c
+        # but if /96.0 is used the font is too small
+        ctx.set_font_size(prop.get_size_in_points() * self.dpi / 72)
 
         y_bearing, w, h = ctx.text_extents(s)[1:4]
         ctx.restore()
@@ -385,37 +308,39 @@ class RendererCairo(RendererBase):
         return w, h, h + y_bearing
 
     def new_gc(self):
+        # docstring inherited
         self.gc.ctx.save()
         self.gc._alpha = 1
-        self.gc._forced_alpha = False # if True, _alpha overrides A from RGBA
+        self.gc._forced_alpha = False  # if True, _alpha overrides A from RGBA
         return self.gc
 
     def points_to_pixels(self, points):
+        # docstring inherited
         return points / 72 * self.dpi
 
 
 class GraphicsContextCairo(GraphicsContextBase):
     _joind = {
-        'bevel' : cairo.LINE_JOIN_BEVEL,
-        'miter' : cairo.LINE_JOIN_MITER,
-        'round' : cairo.LINE_JOIN_ROUND,
-        }
+        'bevel':  cairo.LINE_JOIN_BEVEL,
+        'miter':  cairo.LINE_JOIN_MITER,
+        'round':  cairo.LINE_JOIN_ROUND,
+    }
 
     _capd = {
-        'butt'       : cairo.LINE_CAP_BUTT,
-        'projecting' : cairo.LINE_CAP_SQUARE,
-        'round'      : cairo.LINE_CAP_ROUND,
-        }
+        'butt':        cairo.LINE_CAP_BUTT,
+        'projecting':  cairo.LINE_CAP_SQUARE,
+        'round':       cairo.LINE_CAP_ROUND,
+    }
 
     def __init__(self, renderer):
-        GraphicsContextBase.__init__(self)
+        super().__init__()
         self.renderer = renderer
 
     def restore(self):
         self.ctx.restore()
 
     def set_alpha(self, alpha):
-        GraphicsContextBase.set_alpha(self, alpha)
+        super().set_alpha(alpha)
         _alpha = self.get_alpha()
         rgb = self._rgb
         if self.get_forced_alpha():
@@ -428,11 +353,8 @@ class GraphicsContextCairo(GraphicsContextBase):
         # one for False.
 
     def set_capstyle(self, cs):
-        if cs in ('butt', 'round', 'projecting'):
-            self._capstyle = cs
-            self.ctx.set_line_cap(self._capd[cs])
-        else:
-            raise ValueError('Unrecognized cap style.  Found %s' % cs)
+        self.ctx.set_line_cap(_api.check_getitem(self._capd, capstyle=cs))
+        self._capstyle = cs
 
     def set_clip_rectangle(self, rectangle):
         if not rectangle:
@@ -464,7 +386,7 @@ class GraphicsContextCairo(GraphicsContextBase):
                 offset)
 
     def set_foreground(self, fg, isRGBA=None):
-        GraphicsContextBase.set_foreground(self, fg, isRGBA)
+        super().set_foreground(fg, isRGBA)
         if len(self._rgb) == 3:
             self.ctx.set_source_rgb(*self._rgb)
         else:
@@ -474,24 +396,61 @@ class GraphicsContextCairo(GraphicsContextBase):
         return self.ctx.get_source().get_rgba()[:3]
 
     def set_joinstyle(self, js):
-        if js in ('miter', 'round', 'bevel'):
-            self._joinstyle = js
-            self.ctx.set_line_join(self._joind[js])
-        else:
-            raise ValueError('Unrecognized join style.  Found %s' % js)
+        self.ctx.set_line_join(_api.check_getitem(self._joind, joinstyle=js))
+        self._joinstyle = js
 
     def set_linewidth(self, w):
         self._linewidth = float(w)
         self.ctx.set_line_width(self.renderer.points_to_pixels(w))
 
 
-class FigureCanvasCairo(FigureCanvasBase):
-    supports_blit = False
+class _CairoRegion:
+    def __init__(self, slices, data):
+        self._slices = slices
+        self._data = data
 
-    def print_png(self, fobj, *args, **kwargs):
+
+class FigureCanvasCairo(FigureCanvasBase):
+
+    def copy_from_bbox(self, bbox):
+        surface = self._renderer.gc.ctx.get_target()
+        if not isinstance(surface, cairo.ImageSurface):
+            raise RuntimeError(
+                "copy_from_bbox only works when rendering to an ImageSurface")
+        sw = surface.get_width()
+        sh = surface.get_height()
+        x0 = math.ceil(bbox.x0)
+        x1 = math.floor(bbox.x1)
+        y0 = math.ceil(sh - bbox.y1)
+        y1 = math.floor(sh - bbox.y0)
+        if not (0 <= x0 and x1 <= sw and bbox.x0 <= bbox.x1
+                and 0 <= y0 and y1 <= sh and bbox.y0 <= bbox.y1):
+            raise ValueError("Invalid bbox")
+        sls = slice(y0, y0 + max(y1 - y0, 0)), slice(x0, x0 + max(x1 - x0, 0))
+        data = (np.frombuffer(surface.get_data(), np.uint32)
+                .reshape((sh, sw))[sls].copy())
+        return _CairoRegion(sls, data)
+
+    def restore_region(self, region):
+        surface = self._renderer.gc.ctx.get_target()
+        if not isinstance(surface, cairo.ImageSurface):
+            raise RuntimeError(
+                "restore_region only works when rendering to an ImageSurface")
+        surface.flush()
+        sw = surface.get_width()
+        sh = surface.get_height()
+        sly, slx = region._slices
+        (np.frombuffer(surface.get_data(), np.uint32)
+         .reshape((sh, sw))[sly, slx]) = region._data
+        surface.mark_dirty_rectangle(
+            slx.start, sly.start, slx.stop - slx.start, sly.stop - sly.start)
+
+    @_check_savefig_extra_args
+    def print_png(self, fobj):
         self._get_printed_image_surface().write_to_png(fobj)
 
-    def print_rgba(self, fobj, *args, **kwargs):
+    @_check_savefig_extra_args
+    def print_rgba(self, fobj):
         width, height = self.get_width_height()
         buf = self._get_printed_image_surface().get_data()
         fobj.write(cbook._premultiplied_argb32_to_unmultiplied_rgba8888(
@@ -520,9 +479,9 @@ class FigureCanvasCairo(FigureCanvasBase):
     def print_svgz(self, fobj, *args, **kwargs):
         return self._save(fobj, 'svgz', *args, **kwargs)
 
-    def _save(self, fo, fmt, **kwargs):
+    @_check_savefig_extra_args
+    def _save(self, fo, fmt, *, orientation='portrait'):
         # save PDF/PS/SVG
-        orientation = kwargs.get('orientation', 'portrait')
 
         dpi = 72
         self.figure.dpi = dpi
@@ -554,8 +513,7 @@ class FigureCanvasCairo(FigureCanvasBase):
                     fo = gzip.GzipFile(None, 'wb', fileobj=fo)
             surface = cairo.SVGSurface(fo, width_in_points, height_in_points)
         else:
-            warnings.warn("unknown format: %s" % fmt, stacklevel=2)
-            return
+            raise ValueError("Unknown format: {!r}".format(fmt))
 
         # surface.set_dpi() can be used
         renderer = RendererCairo(self.figure.dpi)
